@@ -546,7 +546,7 @@ func TestExecuteCommand(t *testing.T) {
 		t.Run(tt.name, func(t *testing.T) {
 			mockCmd.On("Execute", tt.cmd, tt.input, tt.executionMode, tt.outputFormat).Return(tt.mockOutput, tt.mockError).Once()
 
-			got, err := executeCommand(tt.cmd, tt.input, tt.executionMode, tt.outputFormat, "")
+			got, err := executeCommand(tt.cmd, tt.input, tt.executionMode, tt.outputFormat, "", false, false)
 			if tt.wantErr {
 				assert.Error(t, err)
 			} else {
@@ -1026,7 +1026,7 @@ func TestExecuteRecipeSimple(t *testing.T) {
 
 	mockCmd.On("Execute", "echo 'Hello'", "", "", "").Return("Hello", nil).Maybe()
 
-	err := evaluateRecipe(recipe, "", map[string]interface{}{}, false)
+	err := evaluateRecipe(recipe, "", map[string]interface{}{})
 	assert.NoError(t, err)
 
 	mockCmd.AssertCalled(t, "Execute", "echo 'Hello'", "", "", "")
